from gettext import gettext as _

from rest_framework import serializers

from pulpcore.plugin import serializers as platform

from . import models


class MavenArtifactSerializer(platform.SingleArtifactContentSerializer):
    """
    A Serializer for MavenArtifact.
    """

    group_id = serializers.CharField(
        help_text=_("Group Id of the artifact's package."),
        read_only=True,
    )
    artifact_id = serializers.CharField(
        help_text=_("Artifact Id of the artifact's package."),
        read_only=True,
    )
    version = serializers.CharField(
        help_text=_("Version of the artifact's package."),
        read_only=True,
    )
    filename = serializers.CharField(
        help_text=_("Filename of the artifact."),
        read_only=True,
    )

    class Meta:
        fields = platform.SingleArtifactContentSerializer.Meta.fields + (
            'group_id', 'artifact_id', 'version', 'filename'
        )
        model = models.MavenArtifact


class MavenRemoteSerializer(platform.RemoteSerializer):
    """
    A Serializer for MavenRemote.

    Add any new fields if defined on MavenRemote.
    Similar to the example above, in MavenArtifactSerializer.
    Additional validators can be added to the parent validators list

    For example::

    class Meta:
        validators = platform.RemoteSerializer.Meta.validators + [myValidator1, myValidator2]
    """

    class Meta:
        fields = platform.RemoteSerializer.Meta.fields
        model = models.MavenRemote


class MavenDistributionSerializer(platform.BaseDistributionSerializer):
    """
    Serializer for Maven Distributions.
    """

    remote = platform.DetailRelatedField(
        required=False,
        help_text=_('Remote that can be used to fetch content when using pull-through caching.'),
        queryset=models.MavenRemote.objects.all(),
        allow_null=True
    )

    class Meta:
        fields = platform.BaseDistributionSerializer.Meta.fields + ('remote',)
        model = models.MavenDistribution



# coding=utf-8
from urllib.parse import urljoin

from pulp_smash.pulp3.constants import (
    BASE_PUBLISHER_PATH,
    BASE_REMOTE_PATH,
    CONTENT_PATH
)

DOWNLOAD_POLICIES = ['on_demand']

MAVEN_CONTENT_NAME = 'maven.artifact'

MAVEN_CONTENT_PATH = urljoin(CONTENT_PATH, 'maven/artifact/')

MAVEN_REMOTE_PATH = urljoin(BASE_REMOTE_PATH, 'maven/maven/')

MAVEN_PUBLISHER_PATH = urljoin(BASE_PUBLISHER_PATH, 'maven/maven/')

MAVEN_FIXTURE_URL = 'https://repo1.maven.org/maven2/'



# Generated by Django 2.2.1 on 2019-05-28 22:33

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('core', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='MavenDistribution',
            fields=[
                ('basedistribution_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.BaseDistribution')),
            ],
            options={
                'abstract': False,
            },
            bases=('core.basedistribution',),
        ),
        migrations.CreateModel(
            name='MavenRemote',
            fields=[
                ('remote_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.Remote')),
            ],
            options={
                'abstract': False,
            },
            bases=('core.remote',),
        ),
        migrations.CreateModel(
            name='MavenArtifact',
            fields=[
                ('content_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.Content')),
                ('group_id', models.CharField(max_length=255)),
                ('artifact_id', models.CharField(max_length=255)),
                ('version', models.CharField(max_length=255)),
                ('filename', models.CharField(max_length=255)),
            ],
            options={
                'unique_together': {('group_id', 'artifact_id', 'version', 'filename')},
            },
            bases=('core.content',),
        ),
    ]



from pulpcore.plugin import viewsets as core

from . import models, serializers


class MavenArtifactFilter(core.ContentFilter):
    """
    FilterSet for MavenArtifact.
    """

    class Meta:
        model = models.MavenArtifact
        fields = [
            'group_id', 'artifact_id', 'version', 'filename'
        ]


class MavenArtifactViewSet(core.ContentViewSet):
    """
    A ViewSet for MavenArtifact.
    """

    endpoint_name = 'artifact'
    queryset = models.MavenArtifact.objects.all()
    serializer_class = serializers.MavenArtifactSerializer
    filterset_class = MavenArtifactFilter


class MavenRemoteViewSet(core.RemoteViewSet):
    """
    A ViewSet for MavenRemote.
    """

    endpoint_name = 'maven'
    queryset = models.MavenRemote.objects.all()
    serializer_class = serializers.MavenRemoteSerializer


class MavenDistributionViewSet(core.BaseDistributionViewSet):
    """
    ViewSet for Maven Distributions.
    """

    endpoint_name = 'maven'
    queryset = models.MavenDistribution.objects.all()
    serializer_class = serializers.MavenDistributionSerializer



# coding=utf-8
"""Utilities for tests for the maven plugin."""
from functools import partial
from unittest import SkipTest

from pulp_smash import api, selectors
from pulp_smash.pulp3.constants import (
    REPO_PATH
)
from pulp_smash.pulp3.utils import (
    gen_remote,
    gen_repo,
    get_content,
    require_pulp_3,
    require_pulp_plugins,
    sync
)

from pulp_maven.tests.functional.constants import (
    MAVEN_CONTENT_NAME,
    MAVEN_CONTENT_PATH,
    MAVEN_FIXTURE_URL,
    MAVEN_REMOTE_PATH,
)


def set_up_module():
    """Skip tests Pulp 3 isn't under test or if pulp_maven isn't installed."""
    require_pulp_3(SkipTest)
    require_pulp_plugins({'pulp_maven'}, SkipTest)


def gen_maven_remote(**kwargs):
    """Return a semi-random dict for use in creating a maven Remote.

    :param url: The URL of an external content source.
    """
    remote = gen_remote(MAVEN_FIXTURE_URL)
    maven_extra_fields = {
        **kwargs
    }
    remote.update(**maven_extra_fields)
    return remote


def get_maven_content_paths(repo):
    """
    Return a list of content in the fixture repo.
    """
    return ["academy/alex/custommatcher/1.0/custommatcher-1.0-javadoc.jar.sha1"]


def get_maven_content_unit_paths(repo):
    """Return the relative path of content units present in a maven repository.

    :param repo: A dict of information about the repository.
    :returns: A list with the paths of units present in a given repository.
    """
    # FIXME: The "relative_path" is actually a file path and name
    # It's just an example -- this needs to be replaced with an implementation that works
    # for repositories of this content type.
    return [
        content_unit['relative_path']
        for content_unit in get_content(repo)[MAVEN_CONTENT_NAME]
    ]


def gen_maven_content_attrs(artifact):
    """Generate a dict with content unit attributes.

    :param: artifact: A dict of info about the artifact.
    :returns: A semi-random dict for use in creating a content unit.
    """
    # FIXME: Add content specific metadata here.
    return {'_artifact': artifact['_href']}


def populate_pulp(cfg, url=MAVEN_FIXTURE_URL):
    """Add maven contents to Pulp.

    :param pulp_smash.config.PulpSmashConfig: Information about a Pulp application.
    :param url: The maven repository URL. Defaults to
        :data:`pulp_smash.constants.MAVEN_FIXTURE_URL`
    :returns: A list of dicts, where each dict describes one file content in Pulp.
    """
    client = api.Client(cfg, api.json_handler)
    remote = {}
    repo = {}
    try:
        remote.update(client.post(MAVEN_REMOTE_PATH, gen_maven_remote(url)))
        repo.update(client.post(REPO_PATH, gen_repo()))
        sync(cfg, remote, repo)
    finally:
        if remote:
            client.delete(remote['_href'])
        if repo:
            client.delete(repo['_href'])
    return client.get(MAVEN_CONTENT_PATH)['results']


skip_if = partial(selectors.skip_if, exc=SkipTest)
"""The ``@skip_if`` decorator, customized for unittest.

:func:`pulp_smash.selectors.skip_if` is test runner agnostic. This function is
identical, except that ``exc`` has been set to ``unittest.SkipTest``.
"""



from gettext import gettext as _
from logging import getLogger
from os import path

from django.db import models

from pulpcore.plugin.models import Content, Remote, BaseDistribution

logger = getLogger(__name__)


class MavenArtifact(Content):
    """
    The Maven artifact content type.

    This content type represents a single file in a Maven repository.
    """

    TYPE = 'artifact'

    group_id = models.CharField(max_length=255, null=False)
    artifact_id = models.CharField(max_length=255, null=False)
    version = models.CharField(max_length=255, null=False)
    filename = models.CharField(max_length=255, null=False)

    class Meta:
        unique_together = ('group_id', 'artifact_id', 'version', 'filename')

    @staticmethod
    def init_from_artifact_and_relative_path(artifact, relative_path):
        """
        Returns an instance of Project for this artifact.

        Args:
            artifact (:class:`~pulpcore.plugin.models.Artifact`): An instance of an Artifact
            relative_path (str): Relative path for the artifact in the Project
        """
        if path.isabs(relative_path):
            raise ValueError(_("Relative path can't start with '/'."))

        group_id, artifact_id, version, filename =\
            MavenArtifact._get_group_artifact_version_filename(relative_path)

        return MavenArtifact(group_id=group_id,
                             artifact_id=artifact_id,
                             version=version,
                             filename=filename)

    @staticmethod
    def _get_group_artifact_version_filename(relative_path):
        """
        Converts a relative path into a tuple of group_id, artifact_id, and version.

        Args:
            relative_path (str): Relative path for the artifact in the repository.

        Returns:
            Tuple (group_id, artifact_id, version, filename)

        """
        sub_path, filename = path.split(relative_path)
        sub_path, version = path.split(sub_path)
        sub_path, artifact_id = path.split(sub_path)
        group_id = sub_path.replace('/', '.')

        return group_id, artifact_id, version, filename


class MavenRemote(Remote):
    """
    A Remote for MavenArtifact.

    Define any additional fields for your new importer if needed.
    """

    TYPE = 'maven'

    @staticmethod
    def get_remote_artifact_content_type(relative_path=None):
        """
        Returns content type that is found at the relative_path.
        """
        return MavenArtifact


class MavenDistribution(BaseDistribution):
    """
    Distribution for 'maven' content.
    """

    TYPE = 'maven'



#!/usr/bin/env python3

from setuptools import find_packages, setup

requirements = [
    'pulpcore-plugin~=0.1rc2',
]

with open('README.rst') as f:
    long_description = f.read()

setup(
    name='pulp-maven',
    version='0.1.0b2',
    description='pulp-maven plugin for the Pulp Project',
    long_description=long_description,
    license='GPLv2+',
    author='Pulp Project Developers',
    author_email='pulp-dev@redhat.com',
    url='http://www.pulpproject.org/',
    python_requires='>=3.6',
    install_requires=requirements,
    include_package_data=True,
    packages=find_packages(exclude=['pulp_maven.tests']),
    classifiers=(
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        'Operating System :: POSIX :: Linux',
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ),
    entry_points={
        'pulpcore.plugin': [
            'pulp_maven = pulp_maven:default_app_config',
        ]
    }
)



# coding=utf-8
"""Tests that verify download of content served by Pulp."""
import hashlib
import unittest
from random import choice
from urllib.parse import urljoin

from pulp_smash import api, config, utils
from pulp_smash.pulp3.constants import BASE_DISTRIBUTION_PATH, REPO_PATH
from pulp_smash.pulp3.utils import (
    gen_distribution,
    gen_repo
)

from pulp_maven.tests.functional.utils import (
    gen_maven_remote,
    get_maven_content_paths
)
from pulp_maven.tests.functional.constants import (
    MAVEN_CONTENT_PATH,
    MAVEN_FIXTURE_URL,
    MAVEN_REMOTE_PATH
)
from pulp_maven.tests.functional.utils import set_up_module as setUpModule  # noqa:F401


MAVEN_DISTRIBUTION_PATH = urljoin(BASE_DISTRIBUTION_PATH, 'maven/maven/')


class DownloadContentTestCase(unittest.TestCase):
    """Verify whether content served by pulp can be downloaded."""

    def test_all(self):
        """Verify whether content served by pulp can be downloaded.

        The process of creating a Maven mirror is simple:

        1. Create a Maven Remote with a URL pointing to the root of a Maven repository.
        2. Create a distribution with the remote set HREF from 1.

        Do the following:

        1. Create a Maven Remote and a Distribution.
        2. Select a random content unit in the distribution. Download that
           content unit from Pulp, and verify that the content unit has the
           same checksum when fetched directly from Maven Central.

        This test targets the following issues:

        * `Pulp #2895 <https://pulp.plan.io/issues/2895>`_
        * `Pulp Smash #872 <https://github.com/PulpQE/pulp-smash/issues/872>`_
        """
        cfg = config.get_config()
        client = api.Client(cfg, api.json_handler)

        repo = client.post(REPO_PATH, gen_repo())
        self.addCleanup(client.delete, repo['_href'])

        body = gen_maven_remote()
        remote = client.post(MAVEN_REMOTE_PATH, body)
        self.addCleanup(client.delete, remote['_href'])

        repo = client.get(repo['_href'])

        # Create a distribution.
        body = gen_distribution()
        body['remote'] = remote['_href']
        response_dict = client.post(MAVEN_DISTRIBUTION_PATH, body)
        dist_task = client.get(response_dict['task'])
        distribution_href = dist_task['created_resources'][0]
        distribution = client.get(distribution_href)
        self.addCleanup(client.delete, distribution['_href'])

        # Pick a content unit, and download it from both Pulp Fixtures…
        unit_path = choice(get_maven_content_paths(repo))
        fixtures_hash = hashlib.sha256(
            utils.http_get(urljoin(MAVEN_FIXTURE_URL, unit_path))
        ).hexdigest()

        # …and Pulp.
        client.response_handler = api.safe_handler

        unit_url = cfg.get_hosts('api')[0].roles['api']['scheme']
        unit_url += '://' + distribution['base_url'] + '/'
        unit_url = urljoin(unit_url, unit_path)

        pulp_hash = hashlib.sha256(client.get(unit_url).content).hexdigest()
        self.assertEqual(fixtures_hash, pulp_hash)

        # Check that Pulp created a MavenArtifact
        content_filter_url = MAVEN_CONTENT_PATH + '?filename=custommatcher-1.0-javadoc.jar.sha1'
        content_unit = client.get(content_filter_url)
        self.assertEqual(1, content_unit.json()['count'])
